#pragma rtGlobals=1		// Use modern global access method.
#include "AR(1)"

//******IMPORTANT: THIS PROCEDURE REQUIRES THAT THE StatFuncs XOP IS LOADED.

//******THE MatrixEig XOP AVAILABLE AT 
//******http://hoshi-o.physiology.uiowa.edu/~avdonin/Siml/IgorPack.html 
//******IS RECOMMENDED: THE BUILT IN MatrixEigenV IS NOT OPTIMIZED FOR THESE MATRICES

//References:

//AS=Allen, M.R., and Smith, L.A., 1996, Monte Carlo SSA: detecting irregular 
//oscillations in the presence of coloured noise, Journal of Climate, 9:3373-3404

//BK=Broomhead, D.S., and King, G., 1986, On the qualitative analysis of experimental 
//dynamical systems, in Sarkar, S., (ed.), Nonlinear Phenomena and Chaos, Bristol 
//(Adam Hilger): 113-144.

//VG=Vautard, R., and Ghil, M, 1989, Singular Spectrum Analysis in nonlinear dynamics 
//with applications to paleoclimatic time series, Physica D, 35:395-424.

//These functions follow procedures outlined in AS, except where otherwise noted.
//Specific sections are referred to as AS(X.Y).

Function SSA_EOF(N,M,ends,dw,Cw,Lw,Ew)
	variable N,M,ends;wave/d dw,Cw,Lw,Ew
	
//Generates empirical orthogonal functions (EOFs).  See SSA() for input annotation.
	
	string ExecStr
	make/d/o ME_Val,ME_Vec,SSA_EOF_tempEigIndex
								//initialize result waves to be accessed later
	redimension/n=(M,M) Ew
	redimension/n=(M) Lw,SSA_EOF_tempEigIndex
	if(ends>0)	//Follow VG recipe for generating lag-covariance matrix.
		VGmat(dw,Cw,M)
	else			//Follow BK recipe for generating lag-covariance matrix.
		BKmat(dw,Cw,M)
	endif
	sprintf ExecStr,"MatrixEig %s",GetWavesDataFolder(Cw,2)
	Execute/Z ExecStr
	if(v_flag)							//Use slower built-in routine if MatrixEig isn't loaded
		MatrixEigenV/L Cw
		wave/c W_eigenvalues;wave M_L_eigenvectors
		Lw=real(W_eigenvalues)
		MakeIndex/R Lw,SSA_EOF_tempEigIndex
		Lw[]=real(W_eigenvalues[SSA_EOF_tempEigIndex[p]])
		Ew[][]=M_L_eigenvectors[p][SSA_EOF_tempEigIndex[q]]
	else
		Lw=ME_Val	//Get eigenvalues and EOFs.
		MakeIndex/R Lw,SSA_EOF_tempEigIndex		//Make index to sort EOFs by eigenvalue magnitude.
		Lw[]=ME_Val[SSA_EOF_tempEigIndex[p]]
		Ew[][]=ME_Vec[p][SSA_EOF_tempEigIndex[q]]	//Sort.
	endif
	killwaves ME_Val,ME_Vec,SSA_EOF_tempEigIndex,W_eigenvalues,M_L_eigenvectors
end

Function Noise_EOF(N,M,dw,K,Cdw,Ldw,Edw,Cnw,Lnw,Enw,Ldw_prime,NDind)
	variable N,M;wave K,NDind
	wave/d dw,Cdw,Ldw,Edw,Cnw,Lnw,Enw,Ldw_prime

	struct AR1_params AR1p
	
	variable i,norm;string ExecStr
	make/d/o/n=(M,M) MakeNoise_tempARparams
	redimension/n=(M,M) Cnw,Enw
	make/d/o/n=(M) ME_Val,ME_Vec,EigIndex,Noise_EOF_tempEOFs
	redimension/n=(M) Lnw,Ldw_prime,NDind
	wave/d EOFn=Noise_EOF_tempEOFs
	wave/d ResMat=MakeNoise_tempARparams
	getARparams(dw,K,AR1p,Cdw,Edw)			//Call procedure to find AR(1) process parameters
	Cnw[][]=AR1p.gamm^(abs(p-q))-AR1p.mu2				//Cnw now contains W
	make/d/o/n=(M,M) M_product
	IdentityMat(M_product);M_product[][]*=K	[p]	//make diagonal matrix containing K
	MatrixMultiply Edw,M_product,Edw/T			//M_product now contains Q
	MatrixMultiply M_product,Cnw,M_product
	Cnw[][]=AR1p.c0*M_product[p][q]					//Cnw now contains c0*QWQ
	norm=1
	if(sum(K,-inf,inf)/M<1)
		norm=10^100
		for(i=0;i<M;i+=1)
			if(K[i]==0)
				norm=min(norm,Ldw[i])		//set norm to the smallest "signal" eigenvalue to avoid degeneracy
			endif
		endfor
		sprintf ExecStr,"MatrixEig %s",GetWavesDataFolder(Cnw,2)
		Execute/Z ExecStr
		if(v_flag)
			MatrixEigenV Cnw
			wave/c W_eigenvalues
			Lnw[]=real(W_eigenvalues[p])
		else
			Lnw[]=ME_Val[p]
		endif
		wavestats/q Lnw;norm*=0.9/V_max	//create normalization factor, following the matlab routine by Eric Breitenberger
	endif
	IdentityMat(M_product);M_product[][]*=(1-K[p])
	MatrixMultiply Edw,M_product,Edw/T			//M_product now contains S
	MatrixMultiply M_product,Cdw,M_product		//M_product now contains SCd
	Cnw[][]=norm*Cnw[p][q]+M_product[p][q]	//Cnw now contains Cn with dampened "noise" eigenvalues
	sprintf ExecStr,"MatrixEig %s",GetWavesDataFolder(Cnw,2)
	Execute/Z ExecStr
	if(v_flag)
		MatrixEigenV/L Cnw
		wave/c W_eigenvalues;wave M_L_eigenvectors
		Cnw[][]=(Cnw[p][q]-M_product[p][q])/norm+M_product[p][q]
		Lnw[]=real(W_eigenvalues[p]/norm)
		MakeIndex/R Lnw,EigIndex
		Lnw[]=real(W_eigenvalues[EigIndex[p]])
		Enw[][]=M_L_eigenvectors[p][EigIndex[q]]
	else
		Cnw[][]=(Cnw[p][q]-M_product[p][q])/norm+M_product[p][q]	//Cnw now contains Cn with no dampening
		Lnw[]=ME_Val[p]/norm			//Get eigenvalues and renormalize
		MakeIndex/R Lnw,EigIndex						//Make index to sort EOFs by eigenvalue magnitude.
		Lnw[]=ME_Val[EigIndex[p]]		//Sort Eigenvalues
		Enw[][]=ME_Vec[p][EigIndex[q]]	//Sort EOFs
	endif
	MatrixMultiply Edw/T,Enw						//Cross correlate to identify eigenvectors
	for(i=0;i<M;i+=1)
		EOFn[]=abs(M_product[p][i]);wavestats/q EOFn
		NDind[i]=V_maxloc							//Make index for correlation
		if(K[NDind[i]]==0)
			Lnw[i]*=norm					//renormalize "signal" eigenvalues
			if(sign(Enw[0][i])==sign(Edw[0][NDind[i]]))
				EOFn[]=Edw[p][NDind[i]]-Enw[p][i]	//check match of known eigenvectors
			else
				EOFn[]=Edw[p][NDind[i]]+Enw[p][i]	//check match of known eigenvectors if polarity is reversed
			endif
			wavestats/q EOFn
			if(V_max>10^-10)
				printf "WARNING: EOF %g, maximum difference: %g\r",NDind[i]+1,V_max
			endif
		endif
	endfor
	MatrixMultiply Enw/T,Cdw,Enw;Ldw_prime[]=M_product[p][p]
	killwaves M_product,MakeNoise_tempARparams,EigIndex,ME_Vec,ME_Val,EOFn,W_eigenvalues,M_L_eigenvectors
end

Function EOF_sym(Edw,EOFsym)
	wave/d Edw,EOFsym
	
//Identifies whether EOF is symmetric or antisymmetric about center.
	variable i,M=dimsize(Edw,0),PR
	redimension/n=(M) EOFsym
	make/d/o/n=(floor(M/2)) EOF_sym_temp1,EOF_sym_temp2
	wave/d t1=EOF_sym_temp1,t2=EOF_sym_temp2
	for(i=0;i<M;i+=1)
		t1[]=Edw[p][i]				//puts the first half of EOF into t1...
		t2[]=Edw[M-p-1][i]			//...and the second half in reverse order into t2,...
		PR=StatsCorrelation(t1,t2)	//...then calculates R (PT[0])
		if(PR>=0)
			EOFsym[i]=5			//if R is positive, then symmetric; values are for display in graphs
		else
			EOFsym[i]=7			//if R is negative, then antisymmetric
		endif
	endfor
	killwaves t1,t2
end

Function EOF_freq(Ew,EOFfreq,EOFfreqR2)
	wave/d Ew,EOFfreq,EOFfreqR2
	
//Identifies dominant frequency associated with each EOF by maximizing the squared correlation with a sinusoid.

	variable i,j,M=dimsize(Ew,0),npts,lastfreq,himag
	make/d/o/n=(M) EOF_freq_temp,EOF_sin_fit_temp
	wave/d EOF=EOF_freq_temp,sw=EOF_sin_fit_temp
	make/d/o/n=(2*M,M/2) EOF_freq_tempmat
	wave/d mat=EOF_freq_tempmat
	make/d/o/n=3 EOF_freq_Pearson_temp
	wave/d PT=EOF_freq_Pearson_temp
	redimension/n=(M) EOFfreq,EOFfreqR2;EOFfreqR2=0;EOFfreq=0
	make/o W_Extremum
	make/d/o/n=251 EOF_freq_tempph,EOF_freq_tempfreq
	wave/d ph=EOF_freq_tempph,freq=EOF_freq_tempfreq
	wave/c/d freqc=EOF_freq_tempfreq;setscale/P x 0,0.002,"",ph
	for(i=0;i<M;i+=1)
		redimension/n=500 freq;SetScale/P x 0,1,"", freq
		freq=0;freq[0,M-1]=Ew[p][i];EOF[]=Ew[p][i]
		fft freq;freqc=r2polar(freqc);ph[]=imag(freqc[p]);redimension/r freqc
		wavestats/q freq;lastfreq=1/0.002;himag=V_max
		do
			wavestats/q freq
			if(V_max<0.75*himag)
				break
			endif
			lastfreq=V_maxloc/0.002
			Optimize/A=1/X={V_maxloc,ph[lastfreq]}/Q/T={0.01,0.01}/S=(0.002) EOF_sin_fit,PT
			V_max=abs(V_max)	//Take absolute value because of bug in Optimize
			if(V_max>EOFfreqR2[i])
				EOFfreq[i]=abs(W_extremum[0])	//absolute value works because this just means the phase got into the wrong quadrant
				EOFfreqR2[i]=V_max
			endif
			freq[lastfreq]=nan
		while(1)
		if(EOFfreq[i]>0.5)
			printf "Error:  EOF %g frequency went too high, %g\r",i+1,W_extremum[0]
			EOFfreq[i]=0.5
		endif
	endfor
	killwaves/z EOF,sw,mat,PT,W_Extremum,W_OptGradient,freq,ph
end

Function/d EOF_sin_fit(w,freq,ph)
	wave w
	variable freq,ph

//returns the squared correlation between EOF and a sine wave having frequency freq and phase ph.
	
	wave/d EOF=EOF_freq_temp,sw=EOF_sin_fit_temp
	sw[]=sin(2*pi*p*freq+ph)
	return StatsCorrelation(EOF,sw)^2
end

Function SSA_MCsimple(dw,Cw,Ew,ends,loPCT,hiPCT,Prob)
	variable ends,Prob;wave/d dw,Cw,Ew,loPCT,hiPCT

//Generates 10000 surrogate waves of length N containing AR(1) (red) noise with 
//the same variance and lag covariance shape as dw.  Diagonalizes each surrogate as Ew'CnwEw, 
//where Cnw is the lag covariance matrix for the surrogate and Ew is a matrix of eigenvectors.  
//For each diagonal element, the 2.5th and 97.5th percentile values are stored in loPCT and hiPCT.

	struct AR1_params AR1p
	
	variable M=dimsize(Ew,0),i,N=numpnts(dw)
	redimension/n=(M) loPCT,hiPCT
	string surr=uniquename("ARsurrog_",1,0);string Lambda="Lambda_"+stringfromlist(1,surr,"_")
	make/d/o/n=(N,10000) $surr;wave/d sw=$surr
	make/d/o/n=(M,10000) $Lambda;wave/d lw=$Lambda
	getARparams(dw,nullwave,AR1p,Cw,Ew)
	sw[0][]=sqrt(AR1p.alpha^2/(1-AR1p.gamm^2))*gnoise(1)
	sw[1,N-1][]=AR1(sw[p-1][q],AR1p)
	make/d/o/n=(N) ARNoise_tempnoise,ARNoise_tempC,M_product
	wave/d tn=ARNoise_tempnoise,Cnw=ARNoise_tempC
	for(i=0;i<10000;i+=1)
		tn[]=sw[p][i]
		if(ends==1)
			VGmat(tn,Cnw,M)
		else
			BKmat(tn,Cnw,M)
		endif
		MatrixMultiply Ew/T,Cnw,Ew;lw[][i]=M_product[p][p]
	endfor
	redimension/n=10000 tn
	for(i=0;i<M;i+=1)
		tn[]=lw[i][p];sort tn,tn
		loPCT[i]=(tn[249]+tn[250])/2;hiPCT[i]=(tn[9749]+tn[9750])/2
	endfor
	killwaves tn,Cnw,M_product,AR
end

Function SSA_MCapprox(dw,Cw,Ew,K,loPCT,hiPCT,Prob)
	wave/d dw,Cw,Ew,loPCT,hiPCT;wave K;variable Prob
	
//Approximates the Monte Carlo confidence limits for the EOFs contained in Ew, using AR(1) noise
//generated with parameters obtained from dw.  This procedure is described in AS(appendix).

	struct AR1_params AR1p
	
	variable N=numpnts(dw),M=dimsize(Ew,0)
	variable nu=3*N/M
	getARparams(dw,K,AR1p,Cw,Ew)
	make/d/o/n=(M,M) M_product;M_product[][]=AR1p.c0*(AR1p.gamm^(abs(p-q))-AR1p.mu2)
	MatrixMultiply Ew/T,M_product,Ew	//M_product now holds the expected eigenspectrum
	redimension/n=(M) loPCT,hiPCT
	loPCT[]=M_product[p][p];hiPCT=loPCT
	Prob=(1-Prob)/2
	variable lo=chi2(nu,Prob)/nu,hi=chi2(nu,1-Prob)/nu
	loPCT*=lo;hiPCT*=hi
	killwaves/Z AR,M_product
end

static Function/d chi2(nu,P)
	variable nu,P
	
	make/d/o chi_square_fit_coeffs={nu,P}
	optimize/L=0/H=100/T=0.001/Q chi_square_fit,chi_square_fit_coeffs
	killwaves chi_square_fit_coeffs
	return V_minloc
end

Function SSA_PC(N,M,Ew,w,PC)
	variable N,M;wave/d Ew,w,PC
	
	variable i,j,k
	redimension/n=(N-M+1,M) PC;PC=0;wavestats/q w
//	make/d/o/n=(M) tempwave
	for(k=0;k<M;k+=1)
		for(i=0;i<N-M+1;i+=1)
			for(j=0;j<M;j+=1)
				PC[i][k]+=(w[i+j]-v_avg)*Ew[j][k]
//				tempwave[]=w[p+i]-v_avg;MatrixMultiply tempwave/T,Edw
//				PC[i][]=M_Product[q];i+=1
			endfor
		endfor
	endfor
//	killwaves tempwave
end

Function SSA_PC_sig(N,M,Ew,w,PC,Sig)
	variable N,M;wave/d Ew,w,PC,Sig
	
	variable i,j,k
	redimension/n=(N-M+1,M) PC;PC=0;wavestats/q w
	for(k=0;k<M;k+=1)
		if(Sig[k]==0)
			for(i=0;i<N-M+1;i+=1)
				for(j=0;j<M;j+=1)
					PC[i][k]+=(w[i+j]-v_avg)*Ew[j][k]
				endfor
			endfor
		endif
	endfor
end

Function SSA_RC(N,M,Ew,PC,RC)
	variable N,M;wave/d Ew,PC,RC
	
	variable i,j,k
	redimension/n=(N,M) RC;RC=0
	for(k=0;k<M;k+=1)
		for(i=0;i<N;i+=1)
			for(j=max(0,i-N+M);j<min(M,i+1);j+=1)
				RC[i][k]+=PC[i-j][k]*Ew[j][k]
			endfor
		endfor
	endfor
	RC[][]/=min(M,min(p+1,N-p))
end

Function SSA_RC_sig(N,M,Ew,PC,RC,Sig)
	variable N,M;wave/d Ew,PC,RC,Sig
	
	variable i,j,k
	redimension/n=(N,M) RC;RC=0
	for(k=0;k<M;k+=1)
		if(Sig[k]==0)
			for(i=0;i<N;i+=1)
				for(j=max(0,i-N+M);j<min(M,i+1);j+=1)
					RC[i][k]+=PC[i-j][k]*Ew[j][k]
				endfor
			endfor
		endif
	endfor
	RC[][]/=min(M,min(p+1,N-p))
end

static Function VGmat(dw,C,M)
	wave/d dw,C;variable M
	
//Fills M x M wave C with lag covariance matrix for dw, 
//extending the data trajectory matrix beyond the ends of dw.

	variable i,j,k,N=numpnts (dw),V_avg=mean(dw,-inf,inf)
	dw-=V_avg
	redimension/n=(M,M) C;C=0
	for(i=0;i<M;i+=1)
		for(k=0;k<N-abs(i-j);k+=1)
			C[i][j]+=(dw[k])*(dw[k+abs(i-j)])
			//Bypass data trajectory matrix; see Allen & Smith, 1995, section 3.1).
		endfor
	endfor
	C[][1,M-1]=C[abs(p-q)][0]
	C[][]/=(N-abs(p-q))
	dw+=V_avg
end

static Function BKmat(dw,C,M)
	wave/d dw,C;variable M

//Fills M x M wave C with lag covariance matrix for dw, 
//truncating the data trajectory matrix at the ends of dw.
	
	variable N=numpnts(dw),V_avg=mean(dw,-inf,inf)
	dw-=V_avg
	redimension/n=(M,M) C
	make/d/o/n=(N-M+1,M) M_product;M_product[][]=dw[p+q]	//M_product now holds data trajectory matrix.
	MatrixMultiply M_product/T,M_product;C=M_product/(N-M+1)
	dw+=V_avg
	killwaves M_product
end

static Function IdentityMat(mat)
	wave/d mat
	
	variable i
	
	mat=0
	do
		mat[i][i]=1;i+=1
	while(i<dimsize(mat,0))
end

